#include <avr/io.h>
#include "bam_conf.h"
 
.comm bam_values, BAM_RESOLUTION+2	// first element = high timer counter; 2nd = bam bit index

#define CYCLES_TO_SYNC (6+32)
#define sync_temp 31


sync:	
	push sync_temp

	in sync_temp, _SFR_IO_ADDR(TCNT0)
	subi sync_temp, lo8(INT_LATENCY + CYCLES_TO_SYNC + 3 + 3)	// 3 for sync call

	bst sync_temp, 0
	brtc .+0						// 1 cycles delay

	bst sync_temp, 1
	brtc .+0						// 2 cycles delay
	brtc .+0	
0:
	nop
	subi sync_temp, -4				// 4 cycles delay routine
	brmi 0b

	pop sync_temp
	ret

.global TIMER0_OVF_vect
TIMER0_OVF_vect:
	push 16
	in 16, _SFR_IO_ADDR(SREG)
	push ZL
	push ZH
	push 16
	push 17

	ldi ZL, lo8(bam_values)
	ldi ZH, hi8(bam_values)

	ld 17, Z						// load counter
	mov 16, 17
	inc 17
	andi 17, hi8((1<<BAM_RESOLUTION)-1)
	st Z+, 17
	breq bam_start
bam_step:
	and 16, 17
	brne bam_end

	// inc index
	ld 17, Z
	inc 17
	st Z, 17

	add ZL, 17
	adc ZH, 16						// r16 = 0

	ldd ZL, Z+8

	rcall sync
	out _SFR_IO_ADDR(BAM_PORT), ZL

bam_end:
	pop 17
	pop 16
	pop ZH
	pop ZL

	out _SFR_IO_ADDR(SREG), 16
	pop 16

	reti


bam_start:
	st Z+, 17						// r17 = 0

	ld 16, Z+
	ld 17, Z+
	ld ZL, Z
	ldi ZH, BAM_OFF

	rcall sync
	out _SFR_IO_ADDR(BAM_PORT), ZH	// zero cycle to fit bam in 2^RESOLUTION cycles
	out _SFR_IO_ADDR(BAM_PORT), 16	// bit 0
	out _SFR_IO_ADDR(BAM_PORT), 17	// bit 1
	ldi ZH, hi8(bam_values+2+3)		// restore ZH
	out _SFR_IO_ADDR(BAM_PORT), ZL	// bit 2
	ldi ZL, lo8(bam_values+2+3)		// restore ZL
	ld 16, Z+
	out _SFR_IO_ADDR(BAM_PORT), 16	// bit 3

	rjmp .+0
	rjmp .+0
	ldi 17, 2

0:
	ld 16, Z+						// 7; Bit 4 to 6
	out _SFR_IO_ADDR(BAM_PORT), 16	// 8
1:
	mov 16, 17						// 1
	inc 17							// 2
2:
	and 16, 17						// 3
	breq 0b							// 4/5

	cpi 17, 1<<4					// 5; 4 iterations: last bit length = 128
	brsh bam_end					// 7
	rjmp 1b							// 8

.end